@@ -827,7 +827,9 @@ static int cabd_find(struct mscab_decompressor_p *this, unsigned char *buf,
 
 	    /* cause the search to restart after this cab's data. */
 	    offset = caboff + cablen;
-	      
+	    if (!offset)
+		return MSPACK_ERR_DATAFORMAT;
+
 	    /* link the cab into the list */
 	    if (!link) *firstcab = cab;
 	    else link->base.next = (struct mscabd_cabinet *) cab;
@@ -1264,7 +1266,7 @@ static int cabd_sys_read(struct mspack_file *file, void *buffer, int bytes) {
       /* check if we're out of input blocks, advance block counter */
       if (this->d->block++ >= this->d->folder->base.num_blocks) {
 	this->error = MSPACK_ERR_DATAFORMAT;
-	return -1;
+	break;
       }
 
       /* read a block */
