@@ -23,6 +23,10 @@
 #include <stdlib.h>
 #include "cltypes.h"
 
+#define CLI_ISCONTAINED(bb, bb_size, sb, sb_size)	\
+    (bb_size > 0 && sb_size > 0 && sb_size <= bb_size	\
+     && sb >= bb && sb + sb_size <= bb + bb_size && sb + sb_size > bb)
+
 typedef struct bitset_tag {
     unsigned char *bitset;
     unsigned long length;
